//**
//** Event Handling Functions
//**
//** This code only ensures the event should happen and extracts the relevent 
//** information from the event object before handing the real code to
//** the logic functions 
//**

//*
//* dyna_onClick()	- event handler to select a grid caell from a mouse click
//*		(no params)	- uses global even object
//*		returns		- void
//*
function dyna_onClick() {
	var oNewCell = _getAncesterElem(event.srcElement, "td");
	selectCell(oNewCell);
}
	
//*
//* dyna_insRow()	- inserts a row before the currently selected cell
//*		oTable		- the Table to add the Row to
//*		returns		- void
//*
function dyna_insRow(oTable) {
	var oCell = getCurrentCell(oTable);
	var oRow = _getAncesterElem(oCell, "tr")
	insertRow(oTable, oRow)
}
	
//*
//* dyna_expCell()	- expands the coll span of the cell
//*		oTable		- the Table to owning the cell
//*		returns		- void
//*
function dyna_expCell(oTable) {
	var oCell = getCurrentCell(oTable);
	if (oCell != null)
		oCell.colSpan++;
}	

//*
//* dyna_redCell()	- reduce the coll span of the cell
//*		oTable		- the Table to owning the cell
//*		returns		- void
//*
function dyna_redCell(oTable) {
	var oCell = getCurrentCell(oTable);
	if (oCell != null && oCell.colSpan > 1)
		oCell.colSpan--;
}	

//*
//* dyna_insPrompt()	- inserts a prompt cell before the currently selected cell
//*		oTable		- the Table to add the Row to
//*		returns		- void
//*
function dyna_insPrompt(oTable) {
	var oCell = getCurrentCell(oTable);
	if (oCell != null)
		insertPrompt(oTable, oCell);
}
	
	
	
//*
//* dyna_insEdge()	- inserts a edge cell before the currently selected cell
//*		oTable		- the Table to add the Row to
//*		returns		- void
//*
function dyna_insEdge(oTable) {
	var oCell = getCurrentCell(oTable);
	if (oCell != null)
		insertEdge(oTable, oCell);
}
	
	
	
//*
//* dyna_insParam()	- inserts a parameter cell before the currently selected cell
//*		oTable		- the Table to add the Row to
//*		returns		- void
//*
function dyna_insParam(oTable) {
	var oCell = getCurrentCell(oTable);
	if (oCell != null)
		insertParam(oTable, oCell);
}
	
	
//*
//* dyna_delCell()	- deletes the currently selected cell
//*		oTable		- the Table to add the Row to
//*		returns		- void
//*
function dyna_delCell(oTable) {
	var oCell = getCurrentCell(oTable);
	if (oCell != null)
		deleteCell(oTable, oCell);
}
	
	
//*
//* dyna_delRow()	- deletes the currently selected cell
//*		oTable		- the Table to add the Row to
//*		returns		- void
//*
function dyna_delRow(oTable) {
	var oCell = getCurrentCell(oTable);
	var oRow = _getAncesterElem(oCell, "tr")

	if (oRow != null)
		deleteRow(oTable, oRow);
}
	
	
	
	
//*
//* dyna_onDragStart()	- event handler for ondragstart event
//*		(no params)	- uses global even object
//*		returns		- void
//*
function dyna_onDragStart() {
	if (event.srcElement.tagName == 'IMG') {
		var oCell = _getAncesterElem(event.srcElement, "td");
		selectCell(oCell);
		startCellDrag(oCell);
	}
}	
	 
//*
//* dyna_onDragOver()	- event handler for ondragover event
//*		(no params)	- uses global even object
//*		returns		- void
//*
function dyna_onDragOver() {

	var oCell = _getAncesterElem(event.srcElement, "td");
	var oTab = _getAncesterElem(event.srcElement, "table");
	var oCell2 = _getCellFromData(event.dataTransfer.getData("text"));
	var oTab2 = _getAncesterElem(oCell2, "table");
	var oCell3 = null;
	var strBefore;

	if ((oTab != null) && (oTab == oTab2)) {
		if ((oCell == null) || (!oTab.contains(oCell))) {
			// <table> elem threw this event
			
		}
		else {
			// <td> elem threw this event

			// get adjacent cell
			if (isDropBefore(oCell)) {
				strBefore = 'Before';
				oCell3 = prevCell(oCell);	
			}
			else {
				strBefore = 'After';
				oCell3 = nextCell(oCell);	
			}
			
			// if dropping here moves cell
			if (
				 (oCell != oCell2)	// Not dropping on same cell
				 && (oCell2 != oCell3)	// Dropping Cell here causes an effect
			) {
				window.status = 'Drag and drop the cell to move it ' + strBefore + ' this cell';
				event.returnValue = false;
				event.dataTransfer.dropEffect = "move";
			}
		}
	}
		
	var oDiag = document.all('diag');
	if (oDiag != null) {
		oDiag.innerText = 'DragOver: ' + event.srcElement.tagName;
	}
}
	
	
//*
//* dyna_onDragLeave()	- event handler for onDragLeave event
//*		(no params)	- uses global even object
//*		returns		- void
//*
function dyna_onDragLeave() {
	event.dataTransfer.dropEffect = "none";
	event.returnValue = false;
	var oCell = _getAncesterElem(event.srcElement, "td");
	if (canDropOnCell(oCell)) {
		window.status = '';
		event.returnValue = false;
		event.dataTransfer.dropEffect = "none";
	}
}

//*
//* dyna_onDrop()	- event handler for onDrop event
//*		(no params)	- uses global even object
//*		returns		- void
//*
function dyna_onDrop() {
	oTgtCell = _getAncesterElem(event.srcElement, "td");
	if (canDropOnCell(oTgtCell)) {
		window.status = '';
		event.returnValue = false;
		var oSrcCell = _getCellFromData(event.dataTransfer.getData("text"));
		moveCell(oSrcCell, oTgtCell, isDropBefore(oTgtCell));
	}
}

//*
//* startCellDrag()	- start the Drag opperation
//*		oCell		- the cell to drag
//*		returns		- void
//*
function startCellDrag(oCell) {
	window.status = 'Drag and drop the cell to move it';
	event.dataTransfer.setData("text", _makeCellData(oCell));
	event.dataTransfer.effectAllowed = "move";
}	

//*
//* canDropOnCell()	- can this cell be dropped on this cell
//*		oCell		- the cell drop on
//*		returns		- true if the cell can be dropped here
//*
function canDropOnCell(oCell) {
	if (oCell == null)
		return false;

	var oTab = _getAncesterElem(oCell, "table");
	var oCell2 = _getCellFromData(event.dataTransfer.getData("text"));
	var oTab2 = _getAncesterElem(oCell2, "table");
	var oCell3;
	
	if (isDropBefore(oCell)) {
		// Don't assume previous sibling is a <td>
		for (oCell3 = oCell.previousSibling; 
			(oCell3 != null) && (oCell3.tagName != 'TD'); 
			oCell3 = oCell3.previousSibling
		)
			;
	}
	else {
		// Don't assume next sibling is a <td>
		for (oCell3 = oCell.nextSibling; 
			(oCell3 != null) && (oCell3.tagName != 'TD'); 
			oCell3 = oCell3.nextSibling
		)
			;
	}

	
	return (
		(oTab == oTab2)			// Source and Target table are the same
		 && (oCell != oCell2)	// Not dropping on same cell
		 && (oCell2 != oCell3)	// Dropping Cell here causes an effect
	);
}

//*
//* canDropOnRow()	- can this cell be dropped on this row
//*		oRow		- the row dropped on
//*		returns		- true if the cell can be dropped here
//*
function canDropOnRow(oRow) {
	if (oRow == null)
		return false;

	var oTab = _getAncesterElem(oRow, "table");
	var oCell2 = _getCellFromData(event.dataTransfer.getData("text"));
	var oTab2 = _getAncesterElem(oCell2, "table");
	
	// Don't assume previous sibling is a <td>
	for (var oCell3 = oRow.lastChild; 
		(oCell3 != null) && (oCell3.tagName != 'TD'); 
		oCell3 = oCell3.previousSibling
	)
		;
	
	return (
		(oTab == oTab2)			// Source and Target table are the same
		 && (oCell2 != oCell3)	// source Cell isn't last cell in row already
	);
}

//*
//* isDropBefore()	- will we drop before or after this cell
//*		oCell		- the cell drop on
//*		returns		- true if we drop before
//*
function isDropBefore(oCell) {
	return ((event.x - oCell.offsetLeft) < (oCell.offsetWidth / 2));
}

//**
//** Logic functions
//**
//** These functions actually do the work - but don't know anything about
//** the event object
//**
	
//*
//* selectCell()	- change the currently selected cell
//*		oCell		- the cell to select
//*		returns		- void
//*
function selectCell(oCell) {
	// Don't select the row thumb
	if ((oCell == null) || (oCell.tagName != 'TD') || (oCell.className == 'row_thumb'))
		return;
		
	var oTab = _getAncesterElem(oCell, "table");
	if (oTab==null)
		return;
			
	var strCurCellId = "select";
	var oOldCell = getCurrentCell(oTab);
		
	if (oOldCell != null) {
		oOldCell.id = "";
	}
	oCell.id = strCurCellId;
}	

//*
//* getCurrentCell()	- Get the currently selected cell
//*		oTable		- the table of the current cell
//*		returns		- the current cell or null
//*
function getCurrentCell(oTable) {
	var strCurCellId = "select";
	return oTable.all(strCurCellId);
}

//*
//* isSameTable()	- checks if two cells belong to the same table
//*		oSrc	- the source cell
//*		oTgt	- the target cell
//*		returns		- true if both cells come from the same table
//*
function moveCell(oSrc, oTgt) {
	var oSrcTbl = _getAncesterElem(oSrc, "table");
	var oTgtTbl = _getAncesterElem(oTgt, "table");
	
	return (oSrcTbl == oTgtTbl);
}

//*
//* isRowThumbCell()	- checks if a cell is the row thumb
//*		oCell	- the source cell
//*		returns		- true if the cell is the row thumb
//*
function isRowThumbCell(oSrc, oTgt) {
	var oSrcTbl = _getAncesterElem(oSrc, "table");
	var oTgtTbl = _getAncesterElem(oTgt, "table");
	
	return (oSrcTbl == oTgtTbl);
}

//*
//* countRowCells()	- counts the cells in a row
//*		oCell		- the source row
//*		returns		- the number of cells (excluding the row thumb cell)
//*
function countRowCells(oRow) {
	var iCnt = 0;
	// Don't assume next sibling is a <td>
	for (var oTmp = oCell.firstChild; oTmp != null; oTmp = oTmp.nextSibling) {
		if ((oTmp.tagName == 'TD') && (oTmp.className != 'thumb_row'))
			iCnt++;
	}
	return iCnt;
}

//*
//* nextCell()	- returns the next Cell in a table
//*		oCell	- the source cell
//*		returns		- the next cell or null
//*
function nextCell(oCell) {
	// Don't assume next sibling is a <td>
	for (var oTmp = oCell.nextSibling; 
		(oTmp != null) && (oTmp.tagName != 'TD'); 
		oTmp = oTmp.nextSibling
	)
		;
	return oTmp;
}

//*
//* prevCell()	- returns the previous Cell in a table
//*		oCell	- the source cell
//*		returns		- the next cell or null
//*
function prevCell(oCell) {
	// Don't assume next sibling is a <td>
	for (var oTmp = oCell.previousSibling; 
		(oTmp != null) && (oTmp.tagName != 'TD'); 
		oTmp = oTmp.previousSibling
	)
		;
	return oTmp;
}

//*
//* insertRow()	- inserts a row
//*		oTable	- the table to add the row to
//*		oRow	- the row to insert before (if null added to the end)
//*		returns		- (void)
//*
function insertRow(oTable, oRow) {
	if (oTable == null)
		return;
	
	var oNewRow;
	if (oRow != null)
		oNewRow = oTable.insertRow(oRow.rowIndex);
	else
		oNewRow = oTable.insertRow();

	// Add row thumb
	var oNewCell = oNewRow.insertCell(0);
	oNewCell.className = "row_thumb";
	oNewCell.innerHTML = "&nbsp;";
}

//*
//* insertPrompt()	- inserts a Prompt Cell
//*		oTable	- the table to add the row to
//*		oCell	- the cell to insert before (if null added to the end)
//*		returns		- (void)
//*
function insertPrompt(oTable, oCell) {
	if (oTable == null)
		return;
	
	var oRow = (oCell==null) ? oTable.rows[0] : _getAncesterElem(oCell, 'tr');
	
	var oNewCell;
	if (oCell != null)
		oNewCell = oRow.insertCell(oCell.cellIndex);
	else
		oNewCell = oRow.insertCell();

	// Add set details
	oNewCell.className = "prompt";
	oNewCell.innerHTML = "<img src=\"pin.gif\">[Prompt ?]";
}

//*
//* insertParam()	- inserts a Param Cell
//*		oTable	- the table to add the row to
//*		oCell	- the cell to insert before (if null added to the end)
//*		returns		- (void)
//*
function insertParam(oTable, oCell) {
	if (oTable == null)
		return;
	
	var oRow = (oCell==null) ? oTable.rows[0] : _getAncesterElem(oCell, 'tr');
	
	var oNewCell;
	if (oCell != null)
		oNewCell = oRow.insertCell(oCell.cellIndex);
	else
		oNewCell = oRow.insertCell();

	// Add set details
	oNewCell.className = "param";
	oNewCell.innerHTML = "<img src=\"pin.gif\">[Param ?]";
}

//*
//* insertEdge()	- inserts a Edge Cell
//*		oTable	- the table to add the row to
//*		oCell	- the cell to insert before (if null added to the end)
//*		returns		- (void)
//*
function insertEdge(oTable, oCell) {
	if (oTable == null)
		return;
	
	var oRow = (oCell==null) ? oTable.rows[0] : _getAncesterElem(oCell, 'tr');
	
	var oNewCell;
	if (oCell != null)
		oNewCell = oRow.insertCell(oCell.cellIndex);
	else
		oNewCell = oRow.insertCell();

	// Add set details
	oNewCell.className = "edge";
	oNewCell.innerHTML = "<img src=\"pin.gif\">[Edge ?]";
}

//*
//* deleteCell()	- deletes a Cell
//*		oTable	- the table to add the row to
//*		oCell	- the cell to delete before
//*		returns		- (void)
//*
function deleteCell(oTable, oCell) {
	if ((oTable == null) || (oCell == null))
		return;
	
	var oRow = _getAncesterElem(oCell, 'tr');
	oRow.deleteCell(oCell.cellIndex);
}

//*
//* deleteRow()	- deletes a Cell
//*		oTable	- the table to add the row to
//*		oRow	- the row to delete before
//*		returns		- (void)
//*
function deleteRow(oTable, oRow) {
	if ((oTable == null) || (oRow == null))
		return;
	
	oTable.deleteRow(oRow.rowIndex);
}

//*
//* moveCell()	- moves a source cell and places it before a target cell
//*		oSrc	- the source cell
//*		oTgt	- the target cell
//*		fBefore	- flag, if true move before target otherwise after
//*		returns		- void
//*
function moveCell(oSrc, oTgt, fBefore) {
	var oSrcRow = _getAncesterElem(oSrc, "tr");
	var oTgtRow = _getAncesterElem(oTgt, "tr");

	var oTmp = oSrc.cloneNode(true);
	
	if (!fBefore) {
		// Don't assume next sibling is a <td>
		for (oTgt = oTgt.nextSibling; 
			(oTgt != null) && (oTgt.tagName != 'TD'); 
			oTgt = oTgt.nextSibling
		)
			;
	}
	
	if (oTgt == null)
		oTgtRow.appendChild(oTmp, oTgt);
	else
		oTgtRow.insertBefore(oTmp, oTgt);
	oSrcRow.removeChild(oSrc);
}
	
//**
//** Private Functions
//**
//** These are small functions - not to be called from outside this file
//** They contain utility functions and functions which encapsulate the 
//** structure of the dyna table out of the logic functions for easier
//** maintanence
//**
	
//*
//* _getAncesterElem() - private function for searching up the DOM tree for  
//*                      a ancester of the specified object with a specified tag
//*                     returns a <td>'s parent <table>
//*		elem	- the DOM object to start with
//*		tag		- the tag to return
//*		returns	- the first occurance of <tag> with in the DOM tree
//*
function _getAncesterElem(elem, tag) {
	for (var tmp = elem; (tmp != null) && (tmp.nodeType == 1) && (tmp.tagName != tag.toUpperCase());tmp = tmp.parentElement)
		;
	return tmp;
}
	
//*
//* _makeCellData() - private function for constucting the pay load for a 
//*                   cell drag and drop  
//*		cell	- the cell DOM object 
//*		returns	- string payload for the cell
//*
function _makeCellData(cell) {
	var oRow = _getAncesterElem(cell, "tr");
	var oTab = _getAncesterElem(cell, "table");
	return oTab.id + " [" + oRow.rowIndex + "," + cell.cellIndex + "]";
}
	
//*
//* _getCellFromData() - private function for retrieve a cell from it's
//*                     pay load for a cell drag and drop  
//*		strPay	- the cell payload 
//*		returns	- DOM cell object
//*
function _getCellFromData(strPay) {
	var rgPatt = /(.*) \[([0-9]+)\,([0-9]+)\]/;
	var arrMatch = strPay.match(rgPatt);
	if (arrMatch==null || arrMatch.length < 4)
		return null;
		 
	var oTab = document.all(arrMatch[1]);
	var iRow = new Number(arrMatch[2]);
	var iCol = new Number(arrMatch[3]);
	
	if ((oTab==null) || isNaN(iRow) || isNaN(iCol))
		return null; 
	
	if (iRow < oTab.rows.length) {
		if (iCol < oTab.rows[iRow].cells.length)
			return oTab.rows[iRow].cells[iCol];
	}
	return null;
}
	

//*
//* _getTableFromCellData() - private function to retrieve a table from it's
//*							pay load for a cell drag and drop  
//*		strPay	- the cell payload 
//*		returns	- DOM cell object
//*
function _getTableFromCellData(strPay) {
	var rgPatt = /(.*) \[([0-9]+)\,([0-9]+)\]/;
	var arrMatch = strPay.match(rgPatt);
	if (arrMatch.length < 4)
		return null;
		 
	return document.all(arrMatch[1]);
}
