//**
//** Event Handling Functions
//**
//** This code only ensures the event should happen and extracts the relevent 
//** information from the event object before handing the real code to
//** the logic functions 
//**

//*
//* dyna_onClick()	- event handler to select a grid cell from a mouse click
//*		(no params)	- uses global even object
//*		returns		- void
//*
function dyna_onClick() {
	var eNewCell = _getAncesterElem(event.srcElement, "td");
	var eTab = _getAncesterElem(eNewCell, "table");
		
	if ((eNewCell==null) || (eTab==null))
		return;
			
	var strCurCellId = "select";
	var eOldCell = eTab.all(strCurCellId);
		
	if (eOldCell != null) {
		eOldCell.id = "";
	}
	eNewCell.id = strCurCellId;
}
	
//*
//* dyna_onDragStart()	- event handler for ondragstart event
//*		(no params)	- uses global even object
//*		returns		- void
//*
function dyna_onDragStart() {
	if (event.srcElement.tagName == 'IMG') {
		var eCell = _getAncesterElem(event.srcElement, "td");
		window.status = 'Drag Started';
		event.dataTransfer.setData("text", _makeCellData(eCell));
		event.dataTransfer.effectAllowed = "move";
	}
}	
	 
//*
//* dyna_onDragEnter()	- event handler for ondragenter event
//*		(no params)	- uses global even object
//*		returns		- void
//*
function dyna_onDragEnter() {
	if (event.srcElement.tagName == 'TD') {
		window.status = 'Can drop here';
		event.returnValue = false;
		event.dataTransfer.dropEffect = "move";
	}
}
	
	
//*
//* dyna_onDragLeave()	- event handler for onDragLeave event
//*		(no params)	- uses global even object
//*		returns		- void
//*
function dyna_onDragLeave() {
	if (event.srcElement.tagName == 'TD') {
		window.status = '';
		event.returnValue = false;
		event.dataTransfer.dropEffect = "none";
	}
}

//*
//* dyna_onDrop()	- event handler for onDrop event
//*		(no params)	- uses global even object
//*		returns		- void
//*
function dyna_onDrop() {
	if (event.srcElement.tagName == 'TD') {
		window.status = '';
		event.returnValue = false;
		//alert("You droped me : " + event.dataTransfer.getData("text"));
		var oCell = _getCellFromData(event.dataTransfer.getData("text"));
		//alert(oCell.innerText);
		moveCell(oCell, event.srcElement);
	}
}
	
//**
//** Logic functions
//**
//** These functions actually do the work - but don't know anything about
//** the event object
//**
	
//*
//* moveCell()	- moves a source cell and places it before a target cell
//*		oSrc	- the source cell
//*		oTgt	- the target cell
//*		returns		- void
//*
function moveCell(oSrc, oTgt) {
	var oSrcRow = _getAncesterElem(oSrc, "tr");
	var oTgtRow = _getAncesterElem(oTgt, "tr");

	var oTmp = oSrc.cloneNode(true);		
	oTgtRow.insertBefore(oTmp, oTgt);
	oSrcRow.removeChild(oSrc);
}
	
//**
//** Private Functions
//**
//** These are small functions - not to be called from outside this file
//** They contain utility functions and functions which encapsulate the 
//** structure of the dyna table out of the logic functions for easier
//** maintanence
//**
	
//*
//* _getAncesterElem() - private function for searching up the DOM tree for  
//*                      a ancester of the specified object with a specified tag
//*                     returns a <td>'s parent <table>
//*		elem	- the DOM object to start with
//*		tag		- the tag to return
//*		returns	- the first occurance of <tag> with in the DOM tree
//*
function _getAncesterElem(elem, tag) {
	for (var tmp = elem; (tmp != null) && (tmp.nodeType == 1) && (tmp.tagName != tag.toUpperCase());tmp = tmp.parentElement)
		;
	return tmp;
}
	
//*
//* _makeCellData() - private function for constucting the pay load for a 
//*                   cell drag and drop  
//*		cell	- the cell DOM object 
//*		returns	- string payload for the cell
//*
function _makeCellData(cell) {
	var oRow = _getAncesterElem(cell, "tr");
	var oTab = _getAncesterElem(cell, "table");
	return oTab.id + " [" + oRow.rowIndex + "," + cell.cellIndex + "]";
}
	
//*
//* _getCellFromData() - private function for retrieve a cell from it's
//*                     pay load for a cell drag and drop  
//*		strPay	- the cell payload 
//*		returns	- DOM cell object
//*
function _getCellFromData(strPay) {
	var rgPatt = /(.*) \[([0-9]+)\,([0-9]+)\]/;
	var arrMatch = strPay.match(rgPatt);
	var strTab = arrMatch[1];
	var strRow = arrMatch[2];
	var strColl = arrMatch[3];
	//alert(strPay + " == " + strTab + " [" + strRow + "," + strColl + "]");
	var oTab = document.all(strTab);
	return oTab.rows(new Number(strRow)).cells(new Number(strColl));
}
	
  